library(tidyverse)
library(here)
library(fs)
library(dplyr)
library(purrr)
library(readxl)

#This script is to output a more user-friendly rosette bottle data file 
#Get sdev data  
sdev <- tibble(files = fs::dir_ls(here("C:/Users/katey/Desktop/CEOS Data Processing/ros/2023_wk_ros_ctd_sn7798/data/10_bottlesum"))) %>% #created a list of files to be imported
  mutate(data = map(files, ~ read_tsv(.x, col_names = FALSE, show_col_types = FALSE))) %>% #imported the files
  mutate(filtered_data = map(data, ~ filter(.x, str_detect(X1, "sdev")))) %>% #filter to keep only rows with "sdev"
  select(filtered_data) %>% #combine all filtered data into a single tibble
  unnest(filtered_data) %>%
  rowid_to_column() %>% #added a column containing row ID
  separate(X1, into = c("Time", "Scan","Dz_dtM","PrdM","DepSM","Tv290C","C0mS_cm","Sal00","Sbeox0V","Par","V0","V1","F0","F1","F2","TimeJ","Spar","Cpar","Latitude","Longitude","value"), sep = " +")  #split data column at white space into multiple columns and assign column names

#Get time data from sdev
time <- sdev %>%
  select(c("rowid", "Time")) #subset time readings as a separate table

#Remove time data from sdev
sdev <- sdev %>% 
  select(-c("Time")) #remove time readings from sdev table 

#Get min data  
min <- tibble(files = fs::dir_ls(here("C:/Users/katey/Desktop/CEOS Data Processing/ros/2023_wk_ros_ctd_sn7798/data/10_bottlesum"))) %>% #created a list of files to be imported
  mutate(data = map(files, ~ read_tsv(.x, col_names = FALSE, show_col_types = FALSE))) %>% #imported the files
  mutate(filtered_data = map(data, ~ filter(.x, str_detect(X1, "min")))) %>% #filter to keep only rows with "min"
  select(filtered_data) %>% #combine all filtered data into a single tibble
  unnest(filtered_data) %>%
  rowid_to_column() %>% #added a column containing row ID
  separate(X1, into = c("Scan","Dz_dtM","PrdM","DepSM","Tv290C","C0mS_cm","Sal00","Sbeox0V","Par","V0","V1","F0","F1","F2","TimeJ","Spar","Cpar","Latitude","Longitude","value"), sep = " +")  #split data column at white space into multiple columns and assign column names

#Get max data  
max <- tibble(files = fs::dir_ls(here("C:/Users/katey/Desktop/CEOS Data Processing/ros/2023_wk_ros_ctd_sn7798/data/10_bottlesum"))) %>% #created a list of files to be imported
  mutate(data = map(files, ~ read_tsv(.x, col_names = FALSE, show_col_types = FALSE))) %>% #imported the files
  mutate(filtered_data = map(data, ~ filter(.x, str_detect(X1, "max")))) %>% #filter to keep only rows with "max"
  select(filtered_data) %>% #combine all filtered data into a single tibble
  unnest(filtered_data) %>%
  rowid_to_column() %>% #added a column containing row ID
  separate(X1, into = c("Scan","Dz_dtM","PrdM","DepSM","Tv290C","C0mS_cm","Sal00","Sbeox0V","Par","V0","V1","F0","F1","F2","TimeJ","Spar","Cpar","Latitude","Longitude","value"), sep = " +")  #split data column at white space into multiple columns and assign column names

#Combine sdev, min, max data
combined_stats <- sdev %>%
  select(-rowid) %>% #exclude row IDs temporarily
  map2_dfc(min %>% select(-rowid), #combine with min data 
           ~ map2_chr(.x, .y, ~ paste0("(", .x, ", ", .y, ")"))) %>%
  map2_dfc(max %>% select(-rowid), #combine with max data
           ~ map2_chr(.x, .y, ~ gsub("\\)", paste0(", ", .y, ")"), .x))) %>%
  mutate(rowid = sdev$rowid, .before = 1) %>%# Reattach row IDs
  select(c("rowid", "Dz_dtM","PrdM","DepSM","Tv290C","C0mS_cm","Sal00","Par","Spar","Cpar","Latitude","Longitude")) %>% #remove unnecessary columns
  rename_with(~ paste0(.x, "_sdevminmax")) %>% #added clarification to column headings 
  rename(rowid = rowid_sdevminmax) #changed rowid column heading back 

#Remove unnecessary data frames
remove(max, min, sdev)

#Get bottle position, date, and avg data
avg <- tibble(files = fs::dir_ls(here("C:/Users/katey/Desktop/CEOS Data Processing/ros/2023_wk_ros_ctd_sn7798/data/10_bottlesum"))) %>% #created a list of files to be imported
  mutate(data = map(files, ~ read_tsv(.x, col_names = FALSE, show_col_types = FALSE))) %>% #imported the files
  mutate(filtered_data = map2(data, files, ~ filter(.x, str_detect(X1, "avg")) %>% #filter to keep only rows with "avg"
                                mutate(source_file = .y))) %>% #add source_file to each filtered dataset
  select(filtered_data) %>% #combine all filtered data into a single tibble
  unnest(filtered_data) %>%
  rowid_to_column() %>% #added a column containing row ID
  separate(X1, into = c("bottle_position","Month","Day","Year","Density00","Sigma_00","Sbeox0ML_L","Sbox0Mm_Kg","Sbeox0PS","Potemp090C","Sva","Scan","Dz_dtM","PrdM","DepSM","Tv290C","C0mS_cm","Sal00","Sbeox0V","Par","V0","V1","F0","F1","F2","TimeJ","Spar","Cpar","Latitude","Longitude","value"), sep = " +") %>% #split data column at white space into multiple columns and assign column names
  select(c("source_file", "rowid", "bottle_position","Month","Day","Year","Density00","Sigma_00","Sbeox0ML_L","Sbox0Mm_Kg","Sbeox0PS","Potemp090C","Sva","Dz_dtM","PrdM","DepSM","Tv290C","C0mS_cm","Sal00","Par", "Spar","Cpar","Latitude","Longitude")) %>% #remove unnecessary columns
  mutate(cast = stringr::str_sub(source_file, -6, -5)) %>% #extracted cast from source file name  
  mutate(cast = as.numeric(cast)) %>% #made column numeric
  select(-c("source_file")) %>% #removed no longer necessary column
  mutate(Month = match(Month,month.abb)) #changed month from MMM (e.g., AUG) to M (e.g., 8)
 
#Read in the logbook
logbook <- read_excel("C:/Users/katey/Desktop/CEOS Data Processing/ros/2023_wk_ros_ctd_sn7798/logbooks/merge_key.xlsx", sheet = "Sheet1", na = "NA") %>% 
  select(Station, Cast)

#Merge CTD data with logbook 
avg <- left_join(avg, logbook, by=c("cast"="Cast"))

#Join data frames
all <- left_join(avg, time, by = "rowid")
all <- left_join(all, combined_stats, by = "rowid")

#Re-arrange columns
all <- all %>%
  select(c("Station","cast","Latitude","Longitude","Year","Month","Day","Time","bottle_position","PrdM","PrdM_sdevminmax","DepSM","DepSM_sdevminmax","Dz_dtM","Dz_dtM_sdevminmax","Tv290C","Tv290C_sdevminmax","Potemp090C","C0mS_cm","C0mS_cm_sdevminmax","Sal00","Sal00_sdevminmax","Density00","Sigma_00","Sva","Sbeox0ML_L","Sbox0Mm_Kg","Sbeox0PS","Par","Par_sdevminmax","Spar","Spar_sdevminmax","Cpar","Cpar_sdevminmax"))

all <- all %>% rename(
  'Cast'=cast,
  'Latitude [Degrees North]'=Latitude,
  'Longitude [Degrees East]'=Longitude,
  'Year (UTC)'=Year,
  'Month (UTC)'=Month,
  'Day (UTC)'=Day,
  'Time (UTC)'=Time,
  'Bottle Position'=bottle_position,
  'Pressure, Strain Gauge [db]'=PrdM,
  'Pressure, Strain Gauge [db] (sdev, min, max)'=PrdM_sdevminmax,
  'Depth [salt water, m]'=DepSM,
  'Depth [salt water, m] (sdev, min, max)'=DepSM_sdevminmax,
  'Descent Rate [m/s]'=Dz_dtM,
  'Descent Rate [m/s] (sdev, min, max)'=Dz_dtM_sdevminmax,
  'Temperature [ITS-90, deg C]'=Tv290C,
  'Temperature [ITS-90, deg C] (sdev, min, max)'=Tv290C_sdevminmax,
  'Potential Temperature [ITS-90, deg C]'=Potemp090C,
  'Conductivity [mS/cm]'=C0mS_cm,
  'Conductivity [mS/cm] (sdev, min, max)'=C0mS_cm_sdevminmax,
  'Salinity, Practical [PSU]'=Sal00,
  'Salinity, Practical [PSU] (sdev, min, max)'=Sal00_sdevminmax,
  'Density [density, kg/m^3]'=Density00,
  'Density [sigma-theta, kg/m^3]'=Sigma_00,
  'Specific Volume Anomaly [10^-8 * m^3/kg]'=Sva,
  'Oxygen, SBE 43 [ml/l]'=Sbeox0ML_L,
  'Oxygen, SBE 43 [umol/kg]'=Sbox0Mm_Kg,
  'Oxygen, SBE 43 [% saturation]'=Sbeox0PS,
  'PAR/Irradiance, Biospherical/Licor [umol photons/m^2/sec]'=Par,
  'PAR/Irradiance, Biospherical/Licor [umol photons/m^2/sec] (sdev, min, max)'=Par_sdevminmax,
  'SPAR, Biospherical/Licor [umol photons/m^2/sec]'=Spar,
  'SPAR, Biospherical/Licor [umol photons/m^2/sec] (sdev, min, max)'=Spar_sdevminmax,
  'CPAR/Corrected Irradiance [%]'=Cpar,
  'CPAR/Corrected Irradiance [%] (sdev, min, max)'=Cpar_sdevminmax)

write.csv(all, "C:/Users/katey/Desktop/CEOS Data Processing/ros/2023_wk_ros_ctd_sn7798/data/11_final/2023_rosette_bottle_data.csv", row.names=FALSE) #saved bottle data as a .CSV file